//
//  A simple symlink test
//

#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>


//
// Creates symlink [new-path] to [old-path], checks it,
// returnes  0 - if everything looks fine and
//  1 - otherwise.
// Usage :
// try-sym [old_path] [new_path]
//

int main (int argc, char * argv [])
{

  char * old_path;
  char * new_path;

  struct stat statold;
  struct stat statbuf;

  int num;
  char * buffer;

  int size = 1;


  if (argc != 3) {
    printf ("Usage: %s [old-path] [new-path]\n", argv[0]);
    return 1;
  }

  old_path = argv [1];
  new_path = argv [2];

  // Create symlink
  if (symlink(old_path, new_path) != 0) {
    perror ("checklink : symlink failed ");
    return 1;
  }

  // stat data of symlink itself
  if (lstat(new_path, &statbuf)== -1){
    perror ("checklink: lstat failed");
    return 1;
  }

  if (! (S_ISLNK (statbuf.st_mode))){
    printf ("checklink : file %s is not a symbol link\n", new_path);
    return 1;
  }


  // Test readlink
  //
  // Increase size of buffer to readlink untile whole symlink body will be read.
  // Check readlink result on every iteration.

  while (1){
    buffer = (char *) malloc (size);
    if (!buffer) {
      perror ("checklink: malloc failed");
      return 1;
    }

    memset (buffer, 0, size);
    num = readlink (new_path, buffer, size);
    if (num < 1 || num > size) {
      perror ("checklink: readlink failed");
      free (buffer);
      return 1;
    }

    // Make sure that readlink did not break things
    if (buffer [num] != 0) {
      printf ("checklink : readlink corrupts memory\n");
      free (buffer);
      return 1;
    }

    // Whole expected symlink body is read
    if (num < size)
      break;

    // Only part of symlink body was read. So we  make a bigger buffer
    // and call `readlink' again.
    free (buffer);
    size *= 2;
  }

}
